/*
 * Copyright (c) 2001-2004 PADL Software Pty Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of PADL Software nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PADL SOFTWARE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL PADL SOFTWARE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>

#include "passfilt.h"

/*
 * This is a sample password filter and notification plugin.
 *
 * Install this with "make configure-ds"; remove with "make unconfigure-ds".
 */

static const char LowerCaseLetters[] = "abcdefghijklmnopqrstuvwxyz";
static const char ArabicNumerals[] = "0123456789";
static const char SpecialCharacters[] = "$!%^";
static const char UpperCaseLetters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

static USHORT MatchPasswordAgainstClass(UNICODE_STRING *Password,
	ULONG classLength,
	const char *classSet)
{
	USHORT i, j, found = 0;

	for (i = 0; i < Password->Length / 2; i++) {
		for (j = 0; j < classLength; j++) {
			if (Password->Buffer[i] == (WCHAR)classSet[j]) {
				found++;
				break;
			}
		}

		if (found)
			break;
	}

	i = 0;
	j = 0;

	return found;
}

/*
 * Called before the password has been changed. Use this to implement
 * password quality checks.
 */
NTSTATUS PasswordFilter(UNICODE_STRING *AccountName,
	UNICODE_STRING *DisplayName,
	UNICODE_STRING *Password,
	BOOLEAN Set)
{
	USHORT classCount = 0;

	if (Password->Length < (6 * sizeof(WCHAR))) {
		return NT_STATUS_PWD_TOO_SHORT;
	}

	classCount += MatchPasswordAgainstClass(Password, sizeof(UpperCaseLetters)  - 1, UpperCaseLetters);
	classCount += MatchPasswordAgainstClass(Password, sizeof(LowerCaseLetters)  - 1, LowerCaseLetters);
	classCount += MatchPasswordAgainstClass(Password, sizeof(ArabicNumerals)    - 1, ArabicNumerals);
	classCount += MatchPasswordAgainstClass(Password, sizeof(SpecialCharacters) - 1, SpecialCharacters);

	if (classCount < 3) {
		return NT_STATUS_ILL_FORMED_PASSWORD;
	}

	return NT_STATUS_OK;
}

/*
 * Called after the password has been changed. Use this API to
 * synchronization with foreign directory services.
 */
NTSTATUS PasswordChangeNotify(UNICODE_STRING *AccountName,
	ULONG RelativeId,
	UNICODE_STRING *NewPassword)
{
	syslog(LOG_DEBUG, "PasswordChangeNotify: RelativeId %08x", RelativeId);

	return NT_STATUS_OK;
}

BOOLEAN InitializeChangeNotify(void)
{
	return TRUE;
}

